@@ -2,7 +2,7 @@
 # -*- coding: utf-8 -*-
 
 # imports.
-import os, sys, syst3m ; sys.path.insert(1, syst3m.defaults.get_source_path(__file__, back=2))
+import os, sys, syst3m ; sys.path.insert(1, syst3m.defaults.source_path(__file__, back=2))
 from ssht00ls.classes.config import *
 import ssht00ls
 
