@@ -145,7 +145,7 @@ class Aliases(object):
 			"json":json_config,
 			"str":config,
 		})
-	def synch(self):
+	def sync(self):
 		if not os.path.exists(f"{HOME}/.ssh"): os.system(f"mkdir {HOME}/.ssh && chown -R {OWNER}:{GROUP} {HOME}/.ssh && chmod 700 {HOME}/.ssh")
 		include = f"include ~/.ssht00ls/lib/aliases"
 		if not os.path.exists(f"{HOME}/.ssh/config"): 
