@@ -13,6 +13,7 @@ class SSync(object):
 		self.exclude = ['__pycache__', '.DS_Store']
 		aliases.sync()
 		self.sync()
+		self.gfp = Formats.FilePath("global")
 	def mount(self, 
 		# the local path.
 		path=None, 
@@ -43,8 +44,8 @@ class SSync(object):
 			return r3sponse.error_response(f"Path [{path}] already exists.")
 		elif "*mounted*" in str(cache.get(id=path, group="daemons")):
 			return r3sponse.error_response(f"Path [{path}] is already mounted.")
-		path = utils.clean_path(path)
-		remote = utils.clean_path(remote)
+		path = self.gfp.clean(path=path)
+		remote = self.gfp.clean(path=remote)
 
 		# pull.
 		response = self.pull(
@@ -149,20 +150,27 @@ class SSync(object):
 					return r3sponse.error_response(f"Path [{path}] is not a directory.")
 
 			# index.
-			index = Files.Dictionary(path=False, dictionary={})
+			index, gfp, ids = Files.Dictionary(path=False, dictionary={}), Formats.FilePath("/"), []
 			for _path_ in Files.Directory(path=path).paths(recursive=True):
 				fp = Formats.FilePath(_path_)
-				id = _path_
-				if fp.directory(): id += " (d)"
-				basepath = fp.base()
-				base = Formats.FilePath(basepath)
-				known = base.exists()
-				if known and basepath not in path:
-					try: index.dictionary[basepath+" (d)"]
-					except: known = False
-				if known:
-					index[base.path+" (d)"] = base.mtime(format="seconds")
-				index[id] = fp.mtime(format="seconds")
+				if fp.name() not in [".DS_Store"] and fp.basename() not in ["__pycache__"]:
+					id = _path_
+					if fp.directory(): id += " (d)"
+					if id not in ids: ids.append(id)
+					index[id] = fp.mtime(format="seconds")
+			def get_dirs(path):
+				dirs = []
+				for _, dirs, _ in os.walk(path):
+					for dir in dirs: 
+						newpath = gfp.clean(path=f"{path}/{dir}/")
+						dirs.append(newpath)
+						dirs += get_dirs(newpath)
+				return dirs
+			for _path_ in get_dirs(path):
+				id = _path_+" (d)"
+				if id not in ids: 
+					index[id] = gfp.mtime(path=_path_, format="seconds")
+					ids.append(id)
 			index.dictionary = index.sort(alphabetical=True)
 
 			# handler.
@@ -236,7 +244,12 @@ class SSync(object):
 
 		# check base.
 		base = Formats.FilePath(path).base(back=1)
-		if not os.path.exists(base): os.system(f"mkdir -p {base}")
+		if not os.path.exists(base): 
+			os.system(f"mkdir -p {base}")
+			if not os.path.exists(base): 
+				return r3sponse.error_response(f"Failed to create pull base {base}.")
+			if log_level >= 3:
+				print(f"Created directory {base}.")
 
 		# pull.
 		if exclude == None: exclude = self.exclude
@@ -244,7 +257,7 @@ class SSync(object):
 		for i in exclude: exclude_str += f"--exclude {i} "
 		delete_str = Formats.Boolean(delete).convert(true="--delete", false="")
 		command = f"rsync -{Formats.Boolean(directory).convert(true='a', false='')}zqt {alias}:{remote} {path} {exclude_str}{delete_str}"
-		loader = syst3m.console.Loader(f"Pulling [{alias}:{remote}] to [{path}]")
+		loader = syst3m.console.Loader(f"Pulling [{alias}:{remote}] to [{path}]", interactive=False)
 		output = syst3m.utils.__execute_script__(command)
 		if len(output) > 0 and output[len(output)-1] == "\n": output = output[:-1]
 		cache.set(id=path, data="*mounted*", group="daemons")
@@ -317,6 +330,8 @@ class SSync(object):
 		base = Formats.FilePath(remote).base(back=1)
 		response = utils.test_ssh_dir(alias=alias, path=base, accept_new_host_keys=accept_new_host_keys, create=True, checks=False)
 		if not response.success: return response
+		if response.created and log_level >= 3:
+				print(f"Created remote directory {base}.")
 
 		# push.
 		if exclude == None: exclude = self.exclude
@@ -324,7 +339,7 @@ class SSync(object):
 		for i in exclude: exclude_str += f"--exclude {i} "
 		delete_str = Formats.Boolean(delete).convert(true="--delete", false="")
 		command = f"rsync -{Formats.Boolean(directory).convert(true='a', false='')}zqt {path} {alias}:{remote} {exclude_str}{delete_str}"
-		loader = syst3m.console.Loader(f"Pushing [{path}] to [{alias}:{remote}].")
+		loader = syst3m.console.Loader(f"Pushing [{path}] to [{alias}:{remote}].", interactive=False)
 		output = syst3m.utils.__execute_script__(command)
 		if len(output) > 0 and output[len(output)-1] == "\n": output = output[:-1]
 		cache.set(id=path, data="*mounted*", group="daemons")
@@ -354,7 +369,7 @@ class SSync(object):
 		# settings.
 		start=True,
 		# the daemons log level.
-		log_level=1,
+		log_level=3,
 		# sandbox (do not delete any files).
 		sandbox=False,
 		# serialized.
@@ -366,7 +381,7 @@ class SSync(object):
 				"remote":None,
 				"path":None,
 				"start":True,
-				"log_level":-1,
+				"log_level":3,
 				"sandbox":False,
 			})
 		path = utils.clean_path(path)
@@ -425,8 +440,6 @@ class SSync(object):
 			# get index.
 			response = self.index(short=True)
 			if not response.success: self.crash(response.error)
-			self.last_index = response.index
-			self.last_remote_index = response.remote_index
 
 			# start.
 			while True:
@@ -483,46 +496,50 @@ class SSync(object):
 			if not remote_response["success"]:  return remote_response
 			response = ssync.index(path=self.path)
 			if not response["success"]:  return response
-			index, remote_index, clean_index, remote_clean_index = response["index"], remote_response["index"], {}, {}
-			all_paths = []
+			index, remote_index, clean_index, remote_clean_index, all_paths = response["index"], remote_response["index"], {}, {}, []
 			for path in list(index.keys()):
 				subpath = self.subpath(path)
-				if subpath not in all_paths: all_paths.append(subpath)
+				if subpath not in all_paths: 
+					all_paths.append(subpath)
+				subpath = subpath.split(" (d)")[0] # unpack after appended to all_paths.
 				clean_index[subpath] = index[path]
 			for path in list(remote_index.keys()):
 				subpath = self.subpath(path)
-				if subpath not in all_paths: all_paths.append(subpath)
+				if subpath not in all_paths: 
+					all_paths.append(subpath)
+				subpath = subpath.split(" (d)")[0] # unpack after appended to all_paths.
 				remote_clean_index[subpath] = remote_index[path]
-			synchronized = clean_index == remote_clean_index
-			#print(f'clean_index: {json.dumps(clean_index,indent=4)}, remote_clean_index: {json.dumps(remote_clean_index,indent=4)}')
+			synchronized, mismatches = self.synchronized(clean_index, remote_clean_index, all_paths)
 			if short in ["automatic", "auto"]:
 				if not synchronized: short = False
 			if short:
+				if self.last_index == {}:
+					self.last_index["0"] = clean_index
+				if self.last_remote_index == {}:
+					self.last_remote_index["0"] = remote_clean_index
 				return r3sponse.success_response(f"Successfully indexed [{self.alias}:{self.path}] & [{self.path}].", {
 					"synchronized":synchronized,
 					"index":index,
 					"remote_index":remote_index,
 				})
-			else: return self.process_index(index, remote_index, all_paths)
+			else: return self.process_index(clean_index, remote_clean_index, mismatches)
 
-		def process_index(self, index, remote_index, paths):
+		def process_index(self, index, remote_index, mismatches):
 
 			# iterate.
 			updates, deletions = {}, {}
-			for path in paths:
+			for subpath, info in mismatches.items():
 
 				# vars.
-				directory = " (d)" in path
-				path = path.split(" (d)")[0]
-				fullpath = self.fullpath(path)
-				try: lmtime = index[fullpath]
+				directory = " (d)" in subpath
+				subpath = subpath.split(" (d)")[0]
+				lfullpath, rfullpath = self.fullpath(subpath), self.fullpath(subpath, remote=True)
+				try: lmtime = index[subpath]
 				except KeyError: lmtime = None
-				try: rmtime = remote_index[fullpath]
+				try: rmtime = remote_index[subpath]
 				except KeyError: rmtime = None
-				try: last_lmtime = self.last_index[fullpath]
-				except KeyError: last_lmtime = None
-				try: last_rmtime = self.last_remote_index[fullpath]
-				except KeyError: last_rmtime = None
+				last_lmtime = self.get_last_index(subpath)
+				last_rmtime = self.get_last_index(subpath, remote=True)
 
 				# wanted vars.
 				local_to_remote, remote_to_local = False, False
@@ -530,34 +547,42 @@ class SSync(object):
 				
 				# should not happen.
 				if rmtime == None and lmtime == None: 
-					self.last_index, self.last_remote_index = index, remote_index
-					return r3sponse.error_response(f"No remote & local modification time present. (path: {path}), (rmtime: {rmtime}), (lmtime: {lmtime}), (last_rmtime: {last_rmtime}), (last_lmtime: {last_lmtime}), full index: {json.dumps(index,indent=4)}.")
+					self.set_last_index(index) ; self.set_last_index(remote_index, remote=True)
+					return r3sponse.error_response(f"No remote & local modification time present. (path: {subpath}), (rmtime: {rmtime}), (lmtime: {lmtime}), (last_rmtime: {last_rmtime}), (last_lmtime: {last_lmtime}), index: {json.dumps(index,indent=4)}, remote index: {json.dumps(remote_index,indent=4)}.")
 
 				# one missing.
 				elif rmtime == None or lmtime == None:
 					
-					# local created a file.
-					if rmtime == None and lmtime != None and last_lmtime == None:
-						local_to_remote = True
+					# remote deleted a file.
+					if rmtime == None and lmtime != None and (last_lmtime != None or last_rmtime != None):
+						if self.log_level >= 3:
+							print(f"Remote deleted a file (path: {subpath}) (rmtime: {rmtime}), (lmtime: {lmtime}), (last_lmtime: {last_lmtime}).")
+						remote_to_local = True
+						options.append("delete")
 
 					# local deleted a file.
 					elif rmtime != None and lmtime == None and last_lmtime != None:
+						if self.log_level >= 3:
+							print(f"Local deleted a file (path: {subpath}) (rmtime: {rmtime}), (lmtime: {lmtime}), (last_lmtime: {last_lmtime}).")
 						local_to_remote = True
 						options.append("delete")
 
+					# local created a file.
+					elif rmtime == None and lmtime != None and last_lmtime == None:
+						if self.log_level >= 3:
+							print(f"Local created a file (path: {subpath}) (rmtime: {rmtime}), (lmtime: {lmtime}), (last_lmtime: {last_lmtime}).")
+						local_to_remote = True
+
 					# remote created a file.
 					elif rmtime != None and lmtime == None and last_lmtime == None:
+						if self.log_level >= 3:
+							print(f"Remote created a file (path: {subpath}) (rmtime: {rmtime}), (lmtime: {lmtime}), (last_lmtime: {last_lmtime}).")
 						remote_to_local = True
 
-					# remote deleted a file.
-					elif rmtime == None and lmtime != None and last_lmtime != None:
-						remote_to_local = True
-						options.append("delete")
-
 					# should not happen.
 					else:
-						self.last_index, self.last_remote_index = index, remote_index
-						return r3sponse.error_response(f"Should not happen (#3243443). (path: {path}), (rmtime: {rmtime}), (lmtime: {lmtime}), (last_rmtime: {last_rmtime}), (last_lmtime: {last_lmtime}), full index: {json.dumps(index,indent=4)}.")
+						self.set_last_index(index) ; self.set_last_index(remote_index, remote=True)
+						return r3sponse.error_response(f"Should not happen (#3243443). (path: {subpath}), (rmtime: {rmtime}), (lmtime: {lmtime}), (last_rmtime: {last_rmtime}), (last_lmtime: {last_lmtime}), index: {json.dumps(index,indent=4)}, remote index: {json.dumps(remote_index,indent=4)}.")
 
 				# both present.
 				elif rmtime != None and lmtime != None:
@@ -576,25 +601,25 @@ class SSync(object):
 
 					# should not happen.
 					else:
-						self.last_index, self.last_remote_index = index, remote_index
-						return r3sponse.error_response(f"Unable to compare rmtime: {rmtime} & lmtime: {lmtime}. (path: {path}), (rmtime: {rmtime}), (lmtime: {lmtime}), (last_rmtime: {last_rmtime}), (last_lmtime: {last_lmtime}), full index: {json.dumps(index,indent=4)}.")
+						self.set_last_index(index) ; self.set_last_index(remote_index, remote=True)
+						return r3sponse.error_response(f"Unable to compare rmtime: {rmtime} & lmtime: {lmtime}. (path: {subpath}), (rmtime: {rmtime}), (lmtime: {lmtime}), (last_rmtime: {last_rmtime}), (last_lmtime: {last_lmtime}), index: {json.dumps(index,indent=4)}, remote index: {json.dumps(remote_index,indent=4)}.")
 
 				# should not happen.
 				else:
-					self.last_index, self.last_remote_index = index, remote_index
-					return r3sponse.error_response(f"Should not happen (#27834628). (path: {path}), (rmtime: {rmtime}), (lmtime: {lmtime}), (last_rmtime: {last_rmtime}), (last_lmtime: {last_lmtime}), full index: {json.dumps(index,indent=4)}.")
+					self.set_last_index(index) ; self.set_last_index(remote_index, remote=True)
+					return r3sponse.error_response(f"Should not happen (#27834628). (path: {subpath}), (rmtime: {rmtime}), (lmtime: {lmtime}), (last_rmtime: {last_rmtime}), (last_lmtime: {last_lmtime}), index: {json.dumps(index,indent=4)}, remote index: {json.dumps(remote_index,indent=4)}.")
 
 				# add to updates.
 				if local_to_remote and remote_to_local:
-					self.last_index, self.last_remote_index = index, remote_index
-					return r3sponse.error_response(f"Can not synchronize both remote to local & local to remote (rmtime: {rmtime}) (lmtime: {lmtime}), (last_rmtime: {last_rmtime}) (last_lmtime: {last_lmtime}), full index: {json.dumps(index,indent=4)}.")
+					self.set_last_index(index) ; self.set_last_index(remote_index, remote=True)
+					return r3sponse.error_response(f"Can not synchronize both remote to local & local to remote (rmtime: {rmtime}) (lmtime: {lmtime}), (last_rmtime: {last_rmtime}) (last_lmtime: {last_lmtime}), index: {json.dumps(index,indent=4)}, remote index: {json.dumps(remote_index,indent=4)}.")
 				if "delete" in options: 
 					if self.log_level >= 1: 
 						if local_to_remote:
-							print(f"Deletion required {path} {self.alias}:{path} (rmtime: {rmtime}) (lmtime: {lmtime})")
+							print(f"Deletion required {lfullpath} {self.alias}:{rfullpath} (rmtime: {rmtime}) (lmtime: {lmtime})")
 						else:
-							print(f"Deletion required {self.alias}:{path} {path} (rmtime: {rmtime}) (lmtime: {lmtime})")
-					deletions[path] = {
+							print(f"Deletion required {self.alias}:{rfullpath} {lfullpath} (rmtime: {rmtime}) (lmtime: {lmtime})")
+					deletions[subpath] = {
 						"options":options,
 						"remote_to_local":remote_to_local,
 						"local_to_remote":local_to_remote,
@@ -602,17 +627,17 @@ class SSync(object):
 				elif remote_to_local or local_to_remote:
 					if self.log_level >= 1: 
 						if local_to_remote:
-							print(f"Update required {path} {self.alias}:{path} (rmtime: {rmtime}) (lmtime: {lmtime})")
+							print(f"Update required {lfullpath} {self.alias}:{rfullpath} (rmtime: {rmtime}) (lmtime: {lmtime})")
 						else:
-							print(f"Update required {self.alias}:{path} {path} (rmtime: {rmtime}) (lmtime: {lmtime})")
-					updates[path] = {
+							print(f"Update required {self.alias}:{rfullpath} {lfullpath} (rmtime: {rmtime}) (lmtime: {lmtime})")
+					updates[subpath] = {
 						"options":options,
 						"remote_to_local":remote_to_local,
 						"local_to_remote":local_to_remote,
 						"directory":directory,}
 			# handler.
-			self.last_index, self.last_remote_index = index, remote_index
-			return r3sponse.success_response(f"Successfully indexed [{self.alias}:{self.path}] & [{self.path}].", {
+			self.set_last_index(index) ; self.set_last_index(remote_index, remote=True)
+			return r3sponse.success_response(f"Successfully indexed [{self.alias}:{rfullpath}] & [{lfullpath}].", {
 				"synchronized":len(updates) == 0 and len(deletions) == 0,
 				"updates":updates,
 				"deletions":deletions,
@@ -631,7 +656,7 @@ class SSync(object):
 				safe=False,
 				accept_new_host_keys=True,
 				checks=False,
-				log_level=-1,)
+				log_level=self.log_level,)
 		def remote_to_local(self, path, info, directory=False, forced=False, delete=False):
 			if self.log_level >= 0: print(f"Synchronizing {self.alias}:{path} to {path} (delete: {delete}) (forced: {forced}).")
 			return self.ssync.pull(
@@ -644,7 +669,34 @@ class SSync(object):
 				safe=False,
 				accept_new_host_keys=True,
 				checks=False,
-				log_level=-1,)
+				log_level=self.log_level,)
+		def synchronized(self, clean_index, remote_clean_index, all_paths):
+			mismatches = {}
+			for fullpath in all_paths:
+				subpath = fullpath.split(" (d)")[0]
+				lmtime, rmtime, synchronized = None, None, True
+				try: 
+					lmtime = clean_index[subpath]
+				except KeyError: a=1
+				try:
+					rmtime = remote_clean_index[subpath]
+				except KeyError: a=1
+				if lmtime != None or rmtime != None:
+					synchronized = lmtime == rmtime
+				# skip dirs with not the same mtime but that do both have a mtime.
+				if " (d)" in fullpath and lmtime != None and rmtime != None and lmtime != rmtime:
+					if self.log_level >= 3:
+						print(f"Revert directory {subpath} from synchronized {synchronized} to True")
+					synchronized = True
+				if not synchronized: 
+					mismatches[subpath] = {
+						"path":subpath,
+						"local_mtime":lmtime,
+						"remote_mtime":rmtime,
+					}
+			synchronized = len(mismatches) == 0
+			if self.log_level >= 3: print(f'synchronized: {synchronized} clean_index: {json.dumps(clean_index,indent=4)}, remote_clean_index: {json.dumps(remote_clean_index,indent=4)} mismatches: {json.dumps(mismatches, indent=4)}.')
+			return synchronized, mismatches
 		def subpath(self, fullpath, remote=False, append_slash=False):
 			if remote: path = self.remote
 			else: path = self.path
@@ -695,7 +747,10 @@ class SSync(object):
 			response = self.index(short="auto")
 			if not response.success: return response
 			elif response.synchronized:
-				return r3sponse.success_response(f"Directories [{self.alias}:{self.path}] & [{self.path}] are already synchronized.")
+				if self.log_level >= 3:
+					return r3sponse.success_response(f"Directories [{self.alias}:{self.path}] & [{self.path}] are already synchronized, index: {json.dumps(response.index, indent=4)}, remote index: {json.dumps(response.remote_index, indent=4)}.")
+				else:
+					return r3sponse.success_response(f"Directories [{self.alias}:{self.path}] & [{self.path}] are already synchronized.")
 			updates,deletions = response.unpack(["updates", "deletions"])
 
 			# iterate updates.
@@ -729,9 +784,12 @@ class SSync(object):
 
 			# push deletions.
 			for path, info in deletions.items():
-				remote = info["local_to_remote"] == True
-				response = self.delete(path, remote=remote)
+				if info["remote_to_local"]:
+					response = self.delete(path, remote=False)
+				else:
+					response = self.delete(path, remote=True)
 				if not response.success: return response
+				self.reset_last_index(path, remote="both")
 
 			# check synchronized index.
 			response = self.index(short=True)
@@ -766,11 +824,54 @@ class SSync(object):
 									"local_mtime":lmtime,
 									"remote_mtime":rmtime,
 								}
-					return r3sponse.error_response(f"Failed to synchronize [{self.alias}:{self.remote}] & [{self.path}], failed to update mismatches: {json.dumps(mismatches, indent=4)}].")
+					return r3sponse.error_response(f"Failed to synchronize [{self.alias}:{self.remote}] & [{self.path}], mismatches: {json.dumps(mismatches, indent=4)}].")
 
 			# handler.
 			return r3sponse.success_response(f"Successfully synchronized [{self.alias}:{self.path}] & [{self.path}].")
-
+		def set_last_index(self, index, remote=False, depth=25):
+			if not remote: indexes = self.last_index
+			else: indexes = self.last_remote_index
+			count = len(indexes)
+			if count > depth:
+				new = {}
+				for key,value in indexes.items():
+					key = int(key)
+					if key > 0:
+						new[str(key-1)] = value
+				new[str(depth)] = index
+				if not remote: self.last_index = new
+				else: self.last_remote_index = new
+			else:
+				indexes[str(len(indexes))] = index
+				if not remote: self.last_lmtime = indexes
+				else: self.last_rmtime = indexes
+		def reset_last_index(self, id, remote=False):
+			if remote == "both":
+				self.reset_last_index(id, remote=False)
+				self.reset_last_index(id, remote=True)
+			else:
+				if not remote: indexes = self.last_index
+				else: indexes = self.last_remote_index
+				for depth in list(indexes.keys()):
+					try: 
+						index = indexes[depth]
+						index[id]
+						indexes[depth][id] = None
+					except KeyError: a=1
+				if not remote: self.last_index = indexes
+				else: self.last_remote_index = indexes
+		def get_last_index(self, id, remote=False):
+			if not remote: indexes = self.last_index
+			else: indexes = self.last_remote_index
+			value = None
+			for _, index in indexes.items():
+				try: value = index[id]
+				except KeyError: a=1
+				if value not in ["None", None, "none"]:
+					break
+			if self.log_level >= 3:
+				print(f"Last index ({id}) {value}) (remote: {remote}), indexes: {indexes}.")
+			return value
 # initialized objects.
 ssync = SSync()
 
